; *** S P L A T F O R M   2 6 0 0 ***

; Atari 2600 version of the 2002 Minigames Competition
; 1K winning game by Robin Harbron
; (C)2003 - Thomas Jentzsch

; TODOs:
; + better collision detection
; o improved platform generation (TODO: remove randomness, 2 pixel diff.)
; x animated Bouncy (no space!)
; o scoring (TODO: more points at higher levels)
; + sounds
; + variable x-speed
; + lifes
; + lifes display at left
; + wait at start (firebutton)
; + loose life at 0 points
; + high score
; - show version number at start (0 points before start)
; + limit minimum platform density (table?)
; x no flicker at RESET/next level (level generation is a problem)
; ? visual end of level
; o smooth scrolling
; o scroll in new playfield (TODO: safety platform)
; ? lenghtLst


    processor 6502
    include vcs.h


;===============================================================================
; A S S E M B L E R - S W I T C H E S
;===============================================================================

VERSION         = $0095

BASE_ADR        = $f800

DEBUG           = 0
ILLEGAL         = 1             ; DCP in kernel used!
LARGE_BOUNCY    = 1             ; Bouncy size: 7x12 or 5x8
RANDOM          = 0
RIGHT_SCORE     = 1

MULTI_COLOR     = 0             ; (+12) multiple colors for Bouncy


;===============================================================================
; C O N S T A N T S
;===============================================================================

SCW             = 160

DIGIT_HEIGHT    = 9

NUM_ROWS        = 18
ROW_H           = 8
KERNEL_TOP      = 45                            ; = 45
KERNEL_H        = NUM_ROWS * ROW_H + KERNEL_TOP ; = 189

;total: DIGIT_HEIGHT + 2 + KERNEL_H = 200

  IF LARGE_BOUNCY
BOUNCY_H        = 12
BOUNCY_W        = 7
  ELSE
BOUNCY_H        = 8
BOUNCY_W        = 5
  ENDIF
BOUNCY_X        = SCW/3-BOUNCY_W/2
BOUNCY_Y        = KERNEL_H-1-ROW_H*4

BOUNCY_COL      = $1c                   ; yellow
BOUNCY_COL_STOP = $3a                   ; orange
BOUNCY_COL_BACK = $46                   ; red

SCORE_COLOR     = $0a                   ; white
HISCORE_COLOR   = $46                   ; red

RAND_EOR_8      = $b2
RAND_SEED       = $2e


; difficulty speed parameters:
GRAVITY         = 55
BUMP            = $558
;GRAVITY         = 50
;BUMP            = $510
;GRAVITY         = 45
;BUMP            = $4d0

SPEED_X         = $18
SPEED_MAX_X     = 2
BOUNCY_MIN_X    = 8
BOUNCY_MAX_X    = SCW*3/8-BOUNCY_W/2

; platform generation parameters:
MAX_PLATFORM    = 8-1
FACTOR          = (255+NUM_ROWS)/(NUM_ROWS*2)-2   ; 7.58 = 7 (~1/18/2)
INIT_SPEED      = 2                     ; initial scrolling speed
DENSITY         = $c0*INIT_SPEED        ; initial plattfrom density
DENSITY_MASK    = >DENSITY|>DENSITY/2
NUM_LEVEL       = 10


;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

factor      .byte
factorLo    .byte

tmpVars     ds 9
tmpVar      = tmpVars
tmpVar2     = tmpVars+1
tmpVar3     = tmpVars+2

frameCnt    .byte
random      .byte

xPosLst     ds NUM_ROWS
hmPosLst    ds NUM_ROWS
pfLst       ds NUM_ROWS
lenghtLst   ds NUM_ROWS

ySpeed      ds 2
ySpeedHi    = ySpeed
ySpeedLo    = ySpeed+1

xSpeed      ds 2
xSpeedHi    = xSpeed
xSpeedLo    = xSpeed+1

;cxRowY      .byte               ; 0..NUM_ROWS-1
sound       .byte
levelCnt    .byte

yBouncy     ds 2
yBouncyHi   = yBouncy
yBouncyLo   = yBouncy+1

xBouncy     ds 2
xBouncyHi   = xBouncy
xBouncyLo   = xBouncy+1

scrollIn    .byte
scrollSum   .byte

end_of_reset = .

; variables not resetted for next level:
level       .byte

score       ds 2
scoreHi     = score
scoreLo     = score+1

end_of_init  = .

; variables initialized at start of game:
initVars    = .

saveRandom  .byte
colorP1     .byte               ; cccc111. bits
lifes       .byte
scoreMax     ds 2
scoreMaxHi  = scoreMax
scoreMaxLo  = scoreMax+1
cxRowY      .byte               ; 0..NUM_ROWS-1

NUM_INIT    = . - initVars

  ECHO "*** RAM: ", ., " ***"


;===============================================================================
; M A C R O S
;===============================================================================

  MAC DEBUG_BRK
    IF DEBUG
      brk                         ;
    ENDIF
  ENDM

  MAC BIT_B
    .byte   $24
  ENDM

  MAC BIT_W
    .byte   $2c     ; 4 cylces
  ENDM

  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM

  MAC CHECKPAGE
    IF >. != >{1}
      ECHO ""
      ECHO "ERROR: different pages! (", {1}, ",", ., ")"
      ECHO ""
      ERR
    ENDIF
  ENDM

  MAC DEC2BCD
    IF {1} > 99
      ECHO "ERROR: Value too large for BCD! (", {1}, ")"
      ERR
    ENDIF
    .byte ({1}) / 10 << 4 | ({1}) % 10
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================
    SEG     Bank0

    ORG     BASE_ADR

;---------------------------------------------------------------
Kernel SUBROUTINE
;---------------------------------------------------------------
.ptrScore   = tmpVars       ;..+7
.tmpY       = tmpVars+8
;---------------------------------------
.ptrBouncy  = tmpVars       ;..+1
.yBouncy    = tmpVars+7
.rows       = tmpVars+8     ; == .tmpY !

DrawScreen:
; setup score pointers:
;    ldy     #1
    ldx     #4-1
.loopScore:
    lda     #>Zero
    sta     .ptrScore,x
    sta     .ptrScore+4,x
    dex
    sty     .tmpY
    lda     score,y
    pha
    lsr
    lsr
    lsr
    lsr
    tay
    lda     DigitTbl,y
    sta     .ptrScore,x
    pla
    and     #$0f
    tay
    lda     DigitTbl,y
    sta     .ptrScore+4,x
    ldy     .tmpY
    dey
    dex
    bpl     .loopScore

.waitTim:
    lda     INTIM
    bne     .waitTim
;    sta     VBLANK

;===============================================
    ldy     #DIGIT_HEIGHT
.loopDigits:
    sta     WSYNC
;---------------------------------------
    ldx     #NUM_ROWS           ; 2
    stx     .rows               ; 3
    lda     lifes               ; 3
    sta     GRP0                ; 3         @11
    lda     (.ptrScore+4),y     ; 5
    sta     GRP1                ; 3         @19
  IF RIGHT_SCORE
    lda     (.ptrScore,x)       ; 6             = SLEEP 6
  ENDIF
    lax     (.ptrScore+6),y     ; 5
    lda     (.ptrScore),y       ; 5
    sta     GRP0                ; 3         @38
    lda     (.ptrScore+2),y     ; 5
    sta     GRP0                ; 3         @46
    stx     GRP1                ; 3         @51
    dey                         ; 2
    bne     .loopDigits         ; 2        @55

    sty     GRP1                ; 3
    sty     GRP0                ; 3 =  6

    lda     xBouncy             ; 3
    jsr     XPosObject0         ; 6 =  9    @70/@09

    lda     #<Bouncy+BOUNCY_H-2 ; 2
    sbc     yBouncyHi           ; 3         C=1!
    sta     .ptrBouncy          ; 3
    sec
    sbc     #<Bouncy-KERNEL_H-1-1 ; 2         C=1!
    sta     .yBouncy            ; 3 = 15

    sty     NUSIZ0              ; 3
    lda     #%111               ; 2
    sta     NUSIZ1              ; 3
    sta     VDELP1              ; 3             enable vertical delay
    sta     HMCLR               ; 3 = 14

  IF MULTI_COLOR
    ldy     #BOUNCY_COL_BACK    ; 2
    lda     xSpeed              ; 3
    bmi     .setColor           ; 2
    ldy     #BOUNCY_COL_STOP    ; 2
    ora     xSpeedLo            ; 3
    beq     .setColor           ; 2
  ENDIF
    ldy     #BOUNCY_COL         ; 2
.setColor:
    sty     COLUP0              ; 3 = 10-19

;=====================================================
; right 8 pixels are blanked by black ball!
    ldy     #KERNEL_H-1+2
.loopKernel:                    ;           @29
; bottom platform row:
; reposition next platform (1/2):
    ldx     #ROW_H-2            ; 2
.loopY:                         ;
    lda     #BOUNCY_H-1         ; 2
    dcp     .yBouncy            ; 5 =  7
    sta     WSYNC               ; 3
;---------------------------------------
    bcs     .doDraw             ; 2
    lda     #0                  ; 2
    BIT_W                       ;-1
.doDraw:
    lda     (.ptrBouncy),y      ; 5
    sta     GRP0                ; 3 = 11    @11
    lda     ColTbl-1,x          ; 4
    and     colorP1             ; 3
    sta     COLUP1              ; 3         @21
    dey                         ; 2

    cpy     #(KERNEL_H-KERNEL_TOP)-1+(ROW_H-1); 2
    bcs     .loopY              ; 2

    dex                         ; 2
    bne     .loopY              ; 2= 20   @31

    lda     #BOUNCY_H-1         ; 2
    dcp     .yBouncy            ; 5
    bcs     .doDraw0            ; 2
    txa                         ; 2
    BIT_W                       ;-1
.doDraw0:
    lda     (.ptrBouncy),y      ; 5
    stx     GRP1                ; 3 = 18    @49 (delayed!)

    bit     CXPPMM              ; 3         @52
    bmi     .setcxRowY          ; 2            save *last* row *with* collision
    BIT_W                       ; 1
.setcxRowY:
    sty     cxRowY              ; 3
    dey                         ; 2
    sta     CXCLR               ; 3 = 14    @63

    ldx     .rows               ; 3
    beq     .exitKernel         ; 2
    dex                         ; 2
    stx     .rows               ; 3 = 10

    sta     GRP0                ; 3 =  3    @76
;---------------------------------------
; empty platform row, reposition next platform
    lda     pfLst,x             ; 4
    sta     GRP1                ; 3
    lda     hmPosLst,x          ; 4
    sta     HMP1                ; 3
    and     #$0f                ; 2
    beq     .veryLeft           ; 2
    sec                         ; 2
WaitObject1:
    sbc     #$01                ; 2
    bne     WaitObject1         ; 2
.veryLeft:
    sta.w   RESP1               ; 4         @23..73!
    sta     WSYNC               ; 3
;---------------------------------------
; top platform row (black)
    sta     HMOVE               ; 3
    sta     COLUP1              ; 3 =  6

    lda     #BOUNCY_H-1         ; 2
    dcp     .yBouncy            ; 5
    bcs     .doDraw1            ; 2
    lda     #0                  ; 2
    BIT_W                       ;-1
.doDraw1
    lda     (.ptrBouncy),y      ; 5
    sta     GRP0                ; 3 = 18    @24 (@25, besser 24!)

    dey                         ; 2
    jmp     .loopKernel         ; 3 =  5

;=====================================================
.exitKernel:
    sta     WSYNC               ;           @69
    stx     GRP0
    stx     VDELP1

;    lda     #2
;    sta     VBLANK

;---------------------------------------------------------------
OverScan SUBROUTINE
;---------------------------------------------------------------
    lda     #36-6
    sta     TIM64T

; *** handle collisions: ***
;    lda     #0
;    sta     COLUBK

; calculate old row:
;    sec                         ;       almost doesn't matter
    lda     yBouncyLo           ;       C=1!
    sbc     ySpeedLo
    lda     yBouncyHi
    sbc     ySpeedHi
    sbc     #ROW_H*2+1-1-1      ;       C=0!
;    cmp     cxRowY              ;       was previous row above?
;    bcc     .skipCollision      ;        no, skip
; bugfix:
;    beq     .skipCollision      ;        no, skip
    clc
    sbc     cxRowY              ;       was previous row above?
    bcc     .skipCollision      ;        no, skip

;    lda     #$40
;    sta     COLUBK

    lda     #<BUMP
    sta     ySpeedLo
    lda     #>BUMP
    sta     ySpeedHi

    lda     #$95                ;       -5 points
    jsr     Add16_ScoreNeg
    bcc     .resetScore         ;       score below zero, loose life!

; start bouncing sound:
    lda     #$0c
    sta     AUDC0
    sta     sound

.skipCollision:
    ldy     #-1                 ;       reinitialize collision variable
    sty     cxRowY

; *** play sounds: ***
    lax     sound
    beq     .skipSound
    bpl     .bounce
    dex
    asl
    bpl     .falling
    asl
    beq     .nextLevel
    stx     AUDV0
    lda     #$04
    sta     AUDC0
    txa
    bne     .setAudF0

; goto next level
.nextLevel:
    inc     level                   ;            yes, increase level...

    lda     colorP1                 ;            ...calculate new level color...
    adc     #$2f                    ;           C=1! (color repeats after 16 levels)
;    bcc     .setCol
;    eor     #$10                    ;            new colors in levels 8...
;.setCol:
    sta     colorP1

    lda     random                  ;            ...save new initial random value...
    sta     saveRandom
.startLevel:
    ldx     #<end_of_reset          ;            ...and start new level
    jmp     StartLevel              ; 3

; make falling sound:
.falling:
    bne     .contFalling
; loose one life:
    lsr     lifes
    lsr     lifes               ;       any more lifes?
    bne     .startLevel         ;        yes, restart current level
                                ;        no, game over!!!
    jsr     CheckHigh           ;       new high score?
    bcc     .skipHigh
    stx     scoreMaxLo
    sty     scoreMaxHi
.skipHigh:
    ldx     #0
    stx     scrollIn

.contFalling:
    txa
    lsr
    lsr
    sta     AUDV0
    lda     #$03
    sta     AUDC0
    lda     #$0c
    bne     .setAudF0

;---------------------------------------
.resetScore:                    ;       set score to zero
    lda     #0
    sta     scoreLo
    sta     scoreHi
.looseLife:
    lda     #$b8                ;       start dying sound ($0b..$00)
    sta     sound
.endMoveJmp:
    jmp     .endMove
;---------------------------------------

; make bouncing sound:
.bounce:
    dex
    txa
    sta     AUDV0
    adc     #$13                ;       C=1!
.setAudF0:
    sta     AUDF0
    stx     sound
.skipSound:

; *** check RESET switch: ***
    ldx     #NUM_INIT-3         ;       number of initialized variables
    lsr     SWCHB
    bcc     .doReset            ;       brk!

; *** wait at start of level: ***
    lda     scrollIn
    bmi     .skipWait
    bne     .endMoveJmp
    bit     INPT4
    bmi     .endMoveJmp
    lda     lifes
    beq     .doReset            ;       brk!
    dec     scrollIn            ;
.skipWait:

    bit     sound
    bmi     .endMoveJmp

; *** move Bouncy vertically: ***
;    ldy     #-1
    lda     #-GRAVITY           ;       y = -1!
    ldx     #<ySpeed
    jsr     Add16

    tay
    lda     ySpeedLo
    ldx     #<yBouncy
    jsr     Add16

; check for falling down:
    cmp     #-18
    bcs     .looseLife

;    lda     #BOUNCY_H+ROW_H-3-1      ; -> just above bottom row
;    lda     #KERNEL_H-1              ; -> directly at top
;    lda     #-1
;    sta     yBouncyHi

; *** horizontal movement: ***

; *** move Bouncy horizontally: ***
.doReset = . + 1
; check joystick:
    ldy     #0                  ;       brk!
    lda     #SPEED_X
    bit     SWCHA
    bpl     .addSpeedX
    tya
    bvs     .addSpeedX
    dey
    lda     #-SPEED_X           ;       brakes faster than accelerating
.addSpeedX:
    ldx     #<xSpeed
    jsr     Add16

; friction:
    asl
    ldy     #0
    lda     #SPEED_X/3
    bcs     .subSpeedX
    dey
    lda     #-SPEED_X/3
.subSpeedX:
    jsr     Add16

; check for direction switch due to friction
    bcc     .checkNeg
    dey
.checkNeg:
    iny
    beq     .stopX               ;       friction changed direction, set x-speed to 0

; limit horizontal speed:
    tay                         ;       a = xSpeedHi
    cpy     #SPEED_MAX_X
    bpl     .limitPos
    cpy     #-SPEED_MAX_X
    bpl     .negOk
    ldy     #-SPEED_MAX_X
    sty     xSpeedHi
.limitPos:
    lda     #0
    sta     xSpeedLo
.negOk:

; move:
    lda     xSpeedLo
    ldx     #<xBouncy
    jsr     Add16               ;       y = xSpeedHi

; limit xBouncy to the left:
    cmp     #BOUNCY_MIN_X       ;       a = xBouncyHi
    bcs     .xBouncyOk
    lda     #BOUNCY_MIN_X
    sta     xBouncyHi
    ldy     #0
.stopX
    sty     xSpeedHi
    sty     xSpeedLo
.xBouncyOk:

;-------------------------------------------------

;    lda     #BOUNCY_H-2
;    sta     yBouncy

; *** check for scrolling: ***
;    lda     xBouncy                 ; 3         a=xBouncy!

.endMove:

; *** position score sprites: ***
  IF RIGHT_SCORE
    lda     #38-16+3*6
  ELSE
    lda     #38-16
  ENDIF
;    lda     #159
    jsr     XPosObject0
    sta     WSYNC

    sta     RESBL
    sta     HMCLR
    lda     #$10
    sta     HMBL

    inx
  IF RIGHT_SCORE
    lda     #38+8+3*6
  ELSE
    lda     #38+8
  ENDIF
    jsr     XPosObject

    lda     #%011
    sta     NUSIZ0
    lsr
    sta     NUSIZ1

.waitTim:
    ldx     INTIM
    bne     .waitTim
    beq     MainLoop
; OverScan


;---------------------------------------------------------------
Start SUBROUTINE
;---------------------------------------------------------------
; cart inserted

    cld
    ldx     #NUM_INIT-1

;---------------------------------------------------------------
Reset:
;---------------------------------------------------------------
; RESET pressed

.loopInit:
    lda     InitTbl,x
    sta     initVars,x
    dex
    bpl     .loopInit

    ldx     #end_of_init

;---------------------------------------------------------------
StartLevel:
;---------------------------------------------------------------
; (re)start level
; x contains number of resetted variables

    lda     #0
.clearLoop:
    dex
    txs
    pha
    bne     .clearLoop

;    lda     #$80
;    sta     COLUBK
;    lda     #$04
;    sta     COLUPF

    dex
    stx     CTRLPF
    stx     ENABL

;---------------------------------------------------------------
GameInit SUBROUTINE
;---------------------------------------------------------------
;.factorLo   = tmpVar
;.cntScroll  = tmpVar+2
;.addHi      = tmpVar+3
;.addLo      = tmpVar+4

;    lda     #BOUNCY_Y
;    sta     yBouncy
;    dec     yBouncy

    lda     saveRandom
    sta     random

    lda     #(SCW-8)/INIT_SPEED
    sta     scrollIn

;    lda     #$10
;    sta     scrollSum


  IF DEBUG ;{
    lda     #2
    sta     COLUBK

    lda     #$95                    ; add missing 60 points (total: 100 points)
    jsr     Add16_ScorePos

    ldx     #BOUNCY_X
    ldx     #40
    stx     xBouncy
    ldx     #BOUNCY_Y
    ldx     #89-1
    stx     yBouncy

    lda     #0
    sta     scrollIn

    lda     #$f0
    sta     hmPosLst
    lda     #$ff
    sta     pfLst

    lda     #$f1
    sta     hmPosLst+7
    sta     hmPosLst+8
    sta     hmPosLst+9

    lda     #$c0
    sta     pfLst+7
    sta     pfLst+8
    lda     #$ff
    sta     pfLst+9

;    lda     #$85
;    sta     hmPosLst+10
;    lda     #$ff
;    sta     pfLst+10
;
;    lda     #$42
;    sta     hmPosLst+6
;    lda     #$ff
;    sta     pfLst+6
;




;    lda     #$ff
;    sta     PF1aLst
;    sta     PF2aLst
;    sta     PF0Lst
;    sta     PF2aLst+NUM_ROWS-1
;    sta     PF2aLst+NUM_ROWS-1-4
;
;    lda     #$0c
;    sta     PF0Lst+10
;    lda     #$60
;    sta     PF2aLst+9
;
;    lda     #$03
;    sta     PF2aLst+4
;    lda     #$03
;    sta     PF2aLst+7
;    lda     #$03
;    sta     PF2aLst+8
;    lda     #$63
;    sta     PF2aLst+9
  ELSE ;}
;    lda     #$60                    ; add missing 60 points (total: 100 points)
    .byte   $a9
    DEC2BCD 100-(SCW-8)/4
    jsr     Add16_ScorePos
  ENDIF

;    lda     #$01
;    sta     scoreHi
;    lsr
;    sta     scoreLo
;GameInit

;;    lda     #%100000
;;    sta     CTRLPF
;    lda     #$30
;    sta     HMBL
;    sta     WSYNC
;    sta     HMOVE
;;    lda     #157+3
;;    ldx     #4
;;    jsr     XPosObject
;    sta     WSYNC
;    sta     HMCLR

;---------------------------------------------------------------
MainLoop:
;---------------------------------------------------------------

;---------------------------------------------------------------
VerticalBlank SUBROUTINE
;---------------------------------------------------------------
    lda     #2
    sta     WSYNC
    sta     VSYNC
    sta     WSYNC

    inc     frameCnt

    sta     WSYNC
    lsr
    ldy     #44-5
    sta     WSYNC
    sta     VSYNC
    sty     TIM64T
; VerticalBlank

;---------------------------------------------------------------
GameCalc SUBROUTINE
;---------------------------------------------------------------

; check, if current level is over:
    lda     xBouncy
    bit     sound
    bmi     .skipXScroll
    bit     levelCnt                ;           128 platforms?
    bpl     .contLevel              ;            no, continue with current level
    cmp     #SCW                    ; 2          yes, Bouncy at the very right? (160)
    bcc     .skipXScroll            ;            no, continue, but stop scrolling
    ldx     #$d0                    ;            yes, start sound for finishing the level
    stx     sound                   ;
    bcs     .skipXScroll            ; 3

.contLevel:
    lda     #BOUNCY_MAX_X
    sec
    sbc     xBouncy
    bcc     .doScroll

; *** scroll in new platforms: ***
    ldx     scrollIn
    dex
    bmi     .skipXScroll
;    cpx     #26*2-1
;    bne     .skipSafety         ;   create "safety" plattform at
;    ror     PF2bLst+NUM_ROWS/2  ;    start of each level
;.skipSafety:
; limit level number:
    ldy     level
    cpy     #NUM_LEVEL-1
    bcc     .levelOk
    ldy     #NUM_LEVEL-1
.levelOk:
    lax     DensityTbl,y
    and     #DENSITY_MASK
    tay
    txa
    ldx     #<factor
    jsr     Add16

    lda     #-INIT_SPEED
    dec     scrollIn
    bne     .doScroll2

; initial platform scrolling finished, now show Bouncy:

;    ldx     #124
;    stx     levelCnt

    ldx     #BOUNCY_Y
    stx     yBouncy
;    ldx     #BOUNCY_X
;    BIT_W
.doScroll:
    ldx     #BOUNCY_MAX_X
    stx     xBouncy
.doScroll2:
    jsr     ScrollRight
.skipXScroll:

; *** setup score colors: ***
    jsr     CheckHigh
    ldy     #1                  ;
    ldx     #HISCORE_COLOR
    lda     lifes
    bne     .currentScore       ;           running
    lda     frameCnt            ;           stopped, score switches...
    bpl     .hiScore            ;           ...between current and high score (c=0!)
    clc
.currentScore:
    bcs     .currentScoreHi
    ldx     #SCORE_COLOR
    BIT_W
.hiScore:
    ldy     #<(scoreMax-score)+1
.currentScoreHi:
    stx     COLUP0
    stx     COLUP1
; GameCalc

    jmp     DrawScreen
; MainLoop


HmStartTbl:
    .byte   $da, $ca, $ba, $aa

;***************************************************************
ScrollRight SUBROUTINE
;***************************************************************
;Screen  xPos
;0
;4       -1
;...
;156     151
;160     155

.dPixel     = tmpVar
.dPixel16   = tmpVar+1
.newPixel   = tmpVar+2
.oldSum     = tmpVar+3

    sta     .dPixel
    asl
    asl
    asl
    asl
    sta     .dPixel16

    lda     scrollSum
    sta     .oldSum
    clc
    adc     .dPixel
    and     #$03
    sta     scrollSum
    ror     .newPixel           ;           N = scroll only

  IF (NUM_ROWS & 1) = 0
    bmi     .skipRandom
    jsr     NextRandom
.skipRandom:
  ENDIF

    ldx     #NUM_ROWS-1
.loopScroll:
    bit     .newPixel
    bmi     .checkSmooth

    jsr     NextRandom

    ldy     pfLst,x
    beq     .emptyRow
  IF RANDOM
    pha
  ENDIF
;    tya

; *** count number of pixels set: ***
    lda     lenghtLst,x
    tay
;    ldy     #-1
;.loopBits:
;    iny
;    asl
;    bne     .loopBits
;    tya                         ;       y = 0..7 (was 6!)
    asl
    asl
    adc     xPosLst,x
    sbc     .oldSum             ;       C=0!
;    cmp     #(SCW-4)-4-1-1
    cmp     #(SCW-4)-4-1-1+4
  IF RANDOM
    pla
    bne     .smoothScroll
  ELSE
    bne     .smoothScroll
    lda     random
  ENDIF
;    cmp     RandomTbl,y
    cmp     RandomTbl-1,y
    bcs     .smoothScroll
; enlarge existing platform:
    lda     #-1
    bcc     .setBit

; *** check for new platform: ***
.emptyRow:
    bit     factor
    bmi     .nextLoop
;    lda     .random
    and     #$7f
    cmp     factor
    bcs     .nextLoop

; create new platform:
    ldy     .oldSum
    lda     HmStartTbl,y
    sta     hmPosLst,x
    tya
    adc     #(SCW-4)-1
    sta     xPosLst,x
    lda     #-FACTOR
.setBit:
    adc     factor              ;       C=0!
    sta     factor
    sec
    ror     pfLst,x
    inc     lenghtLst,x

; *** smooth scrolling: ***
.checkSmooth:
    lda     pfLst,x
    beq     .nextLoop

.smoothScroll:
    lda     xPosLst,x
    clc
    adc     .dPixel
    sta     xPosLst,x
    bcs     .xPosOk

; remove left platfrom piece:
    adc     #4
    sta     xPosLst,x
    dec     lenghtLst,x

; adjust factor:
    lda     #0
    asl     pfLst,x
    bne     .addOne             ;        no, add only 1
    lda     #FACTOR-1           ;        yes, add FACTOR and...
    inc     levelCnt            ;        ...increase level length counter
.addOne:
    adc     factor              ;       C=1!
    sta     factor

    lda     hmPosLst,x
    sec
    sbc     #$40
    bvc     .hmCont
    clc
    adc     #$f0+1
    BIT_W
.xPosOk:
    lda     hmPosLst,x
.hmCont
    sec
    sbc     .dPixel16
    bvc     .hmOk
    clc
    adc     #$10-1          ;
.hmOk:
    sta     hmPosLst,x

.nextLoop:
    dex
    bmi     .exitScroll
    jmp     .loopScroll
.exitScroll:

; *** add points: ***
    bit     .newPixel
    bmi     Exit

    lda     #$01            ;       add 1 point

;    lda     level
;    lsr
;    adc     #1

; ScrollRight

;***************************************************************
Add16_ScorePos SUBROUTINE
;***************************************************************
    ldy     #$00
    BIT_W
Add16_ScoreNeg:
    ldy     #$99
Add16_Score:
    ldx     #<score
    sed
Add16:
    clc
    adc     $01,x
    sta     $01,x
    tya
    adc     $00,x
    sta     $00,x
    cld
Exit:
    rts
; Add16_ScorePos


;***************************************************************
NextRandom:
;***************************************************************
    lda     random              ; 3
    lsr                         ; 2
    bcc     .skipEor            ; 2
    eor     #RAND_EOR_8         ; 2
.skipEor:                       ;
    sta     random              ; 3
  IF RANDOM
    eor     yBouncyLo           ; 3
  ENDIF
; NextRandom
    rts


;***************************************************************
XPosObject0 SUBROUTINE
;***************************************************************
    ldx     #0              ; 2
XPosObject:
    sec                     ; 2
    sta     WSYNC           ; 3
;---------------------------------------
WaitObject:
    sbc     #$0f            ; 2
    bcs     WaitObject      ; 2

  CHECKPAGE WaitObject

    eor     #$07            ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    sta     HMP0,x          ; 4
    sta.wx  RESP0,x         ; 5     @23!
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    rts
; XPosObject0


;***************************************************************
CheckHigh SUBROUTINE
;***************************************************************
; carry doesn't matter!
    lax     scoreLo
    sbc     scoreMaxLo
    lda     scoreHi
    tay
    sbc     scoreMaxHi
    rts
; CheckHigh

CodeEnd:


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
;    org     BASE_ADR + $360, $ff

;    align 256
FREE SET . - CodeEnd

InitTbl:
    .byte   RAND_SEED   ; saveRandom
    .byte   $2e         ; colorP1
    .byte   %01010101   ; lives (4 lifes, only three are shown!)
    .byte   >VERSION    ; scoreMaxHi
    .byte   <VERSION    ; scoreMaxLo
    .byte   -1          ; cxRowY

DigitTbl:
    .byte   #<Zero-1,  #<One-1,   #<Two-1,   #<Three-1, #<Four-1
    .byte   #<Five-1,  #<Six-1,   #<Seven-1, #<Eight-1, #<Nine-1

ColTbl:
    .byte   $f0, $f4, $f6, $f8, $fa, $0e;
;    .byte   $00                             ; overlapping!!!

One:
    .byte   %00000000
    .byte   %00000010
    .byte   %00000010
    .byte   %00000010
Seven:
    .byte   %00000000
    .byte   %00000010
    .byte   %00000010
    .byte   %00000010
Four:
    .byte   %00000000
    .byte   %00000010
    .byte   %00000010
    .byte   %00000010
Zero:
    .byte   %00111100
    .byte   %01000010
    .byte   %01000010
    .byte   %01000010
    .byte   %00000000
    .byte   %01000010
    .byte   %01000010
    .byte   %01000010
Three:
    .byte   %00111100
    .byte   %00000010
    .byte   %00000010
    .byte   %00000010
Nine:
    .byte   %00111100
    .byte   %00000010
    .byte   %00000010
    .byte   %00000010
Eight:
    .byte   %00111100
    .byte   %01000010
    .byte   %01000010
    .byte   %01000010
Six:
    .byte   %00111100
    .byte   %01000010
    .byte   %01000010
    .byte   %01000010
Two:
    .byte   %00111100
    .byte   %01000000
    .byte   %01000000
    .byte   %01000000
Five:
    .byte   %00111100
    .byte   %00000010
    .byte   %00000010
    .byte   %00000010
    .byte   %00111100
    .byte   %01000000
    .byte   %01000000
    .byte   %01000000
    .byte   %00111100

  CHECKPAGE One

DataEnd1:

  IF <. < KERNEL_H-1
    ORG (. & (BASE_ADR + $400)) + KERNEL_H-1
  ENDIF

FREE SET FREE + . - DataEnd1
  IF LARGE_BOUNCY
Bouncy:
    .byte   %00111000
    .byte   %01111100
    .byte   %01111100
    .byte   %11000110
    .byte   %10111010
    .byte   %11111110
    .byte   %11111110
    .byte   %11010110
    .byte   %11010110
    .byte   %01111100
    .byte   %01111100
    .byte   %00111000
  ELSE
Bouncy:
    .byte   %01110000
    .byte   %11011000
    .byte   %10101000
    .byte   %11111000
    .byte   %10101000
    .byte   %10101000
    .byte   %11111000
    .byte   %01110000
  ENDIF

DensityTbl:
Y SET DENSITY
  REPEAT NUM_LEVEL
Y SET Y * 15/16
    .byte (<Y & ~DENSITY_MASK) | (>Y & DENSITY_MASK)
  REPEND

RandomTbl:
Y SET MAX_PLATFORM
  REPEAT MAX_PLATFORM
Y SET Y - 1
    .byte (256*Y+(Y+1)/2)/(Y+1)
  REPEND
;    .byte   256/8*7, 256/7*6, 256/6*5, 256/5*4,.byte   256/4*3, 256/3*2, 256/2*1, 256/1*0
;    .byte   256/15*14, 256/14*12, 256/12*10, 256/10*8, 256/8*6, 256/6*4, 256/4*2, 256/2*0
;    .byte   $0

DataEnd2:
;    .byte   "Splatform v0.95 - (C) 2003 Thomas Jentzsch"

FREE SET FREE + BASE_ADR + $3fc - DataEnd2

  ECHO "*** Free ", FREE, " bytes ***"

    org $fffc, 0

    .word   Start
    .word   Reset






